<?php

namespace Drupal\monarch_vue_field\Plugin\Field\FieldWidget;

use Monarch\VueApp;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin base for vue app widgets.
 *
 * @FieldWidget(
 *   id = "vue_app_default",
 *   label = @Translation("No Vue App"),
 *   field_types = {
 *     "vue_app"
 *   },
 * )
 */
class VueAppDefaultWidget extends WidgetBase {

  const APP_NAME = 'no_app';
  const APP_NAME_ATTRIBUTE = 'data-vue-app';
  const APP_LIBRARIES = [];

  /**
   * Add modify the data before it's passed to the app.
   *
   * @param mixed $data
   *   The data stored for the app.
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The field items.
   *
   * @return mixed
   *   The data that will be passed to the app.
   */
  protected function dataAlter($data, FieldItemListInterface $items) {
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['editor'] = VueApp::create(static::APP_NAME, static::APP_LIBRARIES, static::APP_NAME_ATTRIBUTE)
      ->render($this->dataAlter($items->data ?? [], $items));

    $element['data'] = [
      '#type' => 'hidden',
      '#default_value' => '',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    return [['data' => json_decode($values[0]['data'] ?? NULL, TRUE)]];
  }

}
